/*******************************************************
*
*  Total Equilibrium
*  a game for the Mignon Gamekit
*  
*  idea and programming by Daniel Gasser, Switzerland
*  e-mail: gaess@gmx.net
*
*  This game is free software under gpl v2
*
*******************************************************/



#include <gamekit_2_1.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>

// EEMEM wird bei aktuellen Versionen der avr-lib in eeprom.h definiert
// hier: definiere falls noch nicht bekannt ("alte" avr-libc)
#ifndef EEMEM
// alle Textstellen EEMEM im Quellcode durch __attribute__ ... ersetzen
#define EEMEM  __attribute__ ((section (".eeprom")))
#endif


// Globale Variablen definieren. Diese Variablen können wir überall im Programm ansprechen
uint8_t level, noobi, i, j, offset, img_afterboot[5][7];
uint16_t score, hi;
uint16_t allhi EEMEM = 0;

// Bilder definieren
const uint8_t img_boot[5][7] PROGMEM = {
	 0,  0,  0,  0,  0,  0,  0,
	 0, 15, 13,  0, 13, 15,  0,
	 0,  0, 15, 18, 15,  0,  0,
	 0, 15, 13,  0, 13, 15,  0,
	 0,  0,  0,  0,  0,  0,  0,
};

const uint8_t img_end[5][7] PROGMEM = {
	 0, 15,  15,  0, 15, 15,  0,
	 0,  0,  15,  0, 15,  0,  0,
	 0,  0,   0, 15,  0,  0,  0,
	 0,  0,  15,  0, 15,  0,  0,
	 0, 15,  15,  0, 15, 15,  0
};

const uint8_t img_empty[5][7] PROGMEM = {
	 0,  0,  0,  0,  0,  0,  0,
	 0,  0,  0,  0,  0,  0,  0,
	 0,  0,  0,  0,  0,  0,  0,
	 0,  0,  0,  0,  0,  0,  0,
	 0,  0,  0,  0,  0,  0,  0,
};

const uint8_t img_hi[5][7] PROGMEM = {
	15,  0, 15,  0, 15, 15, 15,
	15,  0, 15,  0,  0, 15,  0,
	15, 15, 15,  0,  0, 15,  0,
	15,  0, 15,  0,  0, 15,  0,
	15,  0, 15,  0, 15, 15, 15,
};

const uint8_t img_intro[5][89] PROGMEM = {
	 0,  0,  0,  0,  0,  0,  0, 18, 18, 18, 18,  0,  0, 18, 18,  0,  0, 18, 18, 18, 18,  0,  0, 18, 18,  0,  0,  0, 18, 18,  0,  0,  0,  0,  0,  0, 18, 18, 18,  0,  0, 18, 18,  0,  0, 18,  0, 18,  0, 18, 18, 18,  0, 18,  0,  0,  0, 18, 18, 18,  0, 18, 18,  0,  0, 18, 18,  0,  0, 18, 18, 18,  0, 18,  0, 18,  0, 18,  0,  0,  0, 18,  0,  0,  0,  0,  0,  0,  0,
	 0,  0,  0,  0,  0,  0,  0, 15, 15, 15, 15,  0, 15, 15, 15, 15,  0, 15, 15, 15, 15,  0, 15, 15, 15, 15,  0,  0, 15, 15,  0,  0,  0,  0,  0,  0, 15,  0,  0,  0, 15,  0, 15,  0,  0, 15,  0, 15,  0,  0, 15,  0,  0, 15,  0,  0,  0,  0, 15,  0,  0, 15,  0, 15,  0, 15,  0, 15,  0,  0, 15,  0,  0, 15,  0, 15,  0, 15, 15,  0, 15, 15,  0,  0,  0,  0,  0,  0,  0,
	 0,  0,  0,  0,  0,  0,  0,  0, 15, 15,  0,  0, 15,  0,  0, 15,  0,  0, 15, 15,  0,  0, 15,  0,  0, 15,  0,  0, 15, 15,  0,  0,  0,  0,  0,  0, 15, 15,  0,  0, 15,  0, 15,  0,  0, 15,  0, 15,  0,  0, 15,  0,  0, 15,  0,  0,  0,  0, 15,  0,  0, 15, 15,  0,  0, 15,  0, 15,  0,  0, 15,  0,  0, 15,  0, 15,  0, 15, 15,  0, 15, 15,  0,  0,  0,  0,  0,  0,  0,
	 0,  0,  0,  0,  0,  0,  0,  0, 15, 15,  0,  0, 15, 15, 15, 15,  0,  0, 15, 15,  0,  0, 15, 15, 15, 15,  0,  0, 15, 15, 15, 15,  0,  0,  0,  0, 15,  0,  0,  0, 15,  0, 15,  0,  0, 15,  0, 15,  0,  0, 15,  0,  0, 15,  0,  0,  0,  0, 15,  0,  0, 15,  0, 15,  0, 15, 15,  0,  0,  0, 15,  0,  0, 15, 15, 15,  0, 15,  0, 15,  0, 15,  0,  0,  0,  0,  0,  0,  0,
	 0,  0,  0,  0,  0,  0,  0,  0, 15, 15,  0,  0,  0, 15, 15,  0,  0,  0, 15, 15,  0, 15, 15,  0,  0, 15, 15,  0, 15, 15, 15, 15,  0,  0,  0,  0, 15, 15, 15,  0,  0, 15,  0, 15,  0,  0, 15, 15,  0, 15, 15, 15,  0, 15, 15, 15,  0, 15, 15, 15,  0, 15, 15,  0,  0, 15,  0, 15,  0, 15, 15, 15,  0,  0, 15, 15,  0, 15,  0, 15,  0, 15,  0,  0,  0,  0,  0,  0,  0,
};

const uint8_t img_master[5][39] PROGMEM = {
	 0,  0,  0,  0,  0,  15,  0,  0,  0, 15,  0,  0, 15,  0,  0,  0, 15, 15,  0, 15, 15, 15,  0, 15, 15, 15,  0, 15, 15,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
	 0,  0,  0,  0,  0,  15, 15,  0, 15, 15,  0, 15,  0, 15,  0, 15,  0,  0,  0,  0, 15,  0,  0, 15,  0,  0,  0, 15,  0, 15,  0,  0,  0,  0,  0,  0,  0,  0,  0,
	 0,  0,  0,  0,  0,  15,  0, 15,  0, 15,  0, 15,  0, 15,  0,  0, 15,  0,  0,  0, 15,  0,  0, 15, 15,  0,  0, 15, 15,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
	 0,  0,  0,  0,  0,  15,  0,  0,  0, 15,  0, 15, 15, 15,  0,  0,  0, 15,  0,  0, 15,  0,  0, 15,  0,  0,  0, 15,  0, 15,  0,  0,  0,  0,  0,  0,  0,  0,  0,
	 0,  0,  0,  0,  0,  15,  0,  0,  0, 15,  0, 15,  0, 15,  0, 15, 15, 15,  0,  0, 15,  0,  0, 15, 15, 15,  0, 15,  0, 15,  0,  0,  0,  0,  0,  0,  0,  0,  0,
};

const uint8_t img_levels[63][9] PROGMEM = {
	0,  0,  0, 15,  0,  0,  0,  0,  0,
	0,  0, 15, 15,  0,  0,  0,  0,  0,
	0,  0,  0, 15,  0,  0,  0,  0,  0,
	0,  0,  0, 15,  0,  0,  0,  0,  0,
	0,  0, 15, 15, 15,  0,  0,  0,  0,
	0,  0,  0,  0,  0,  0,  0,  0,  0,
	0,  0,  0,  0,  0,  0,  0,  0,  0,
	0,  0,  0, 15,  0,  0,  0,  0,  0,
	0,  0, 15,  0, 15,  0,  0,  0,  0,
	0,  0,  0,  0, 15,  0,  0,  0,  0,
	0,  0,  0, 15,  0,  0,  0,  0,  0,
	0,  0, 15, 15, 15,  0,  0,  0,  0,
	0,  0,  0,  0,  0,  0,  0,  0,  0,
	0,  0,  0,  0,  0,  0,  0,  0,  0,
	0,  0, 15, 15,  0,  0,  0,  0,  0,
	0,  0,  0,  0, 15,  0,  0,  0,  0,
	0,  0,  0, 15,  0,  0,  0,  0,  0,
	0,  0,  0,  0, 15,  0,  0,  0,  0,
	0,  0, 15, 15,  0,  0,  0,  0,  0,
	0,  0,  0,  0,  0,  0,  0,  0,  0,
	0,  0,  0,  0,  0,  0,  0,  0,  0,
	0,  0,  0,  0, 15,  0,  0,  0,  0,
	0,  0,  0, 15, 15,  0,  0,  0,  0,
	0,  0, 15,  0, 15,  0,  0,  0,  0,
	0,  0, 15, 15, 15,  0,  0,  0,  0,
	0,  0,  0,  0, 15,  0,  0,  0,  0,
	0,  0,  0,  0,  0,  0,  0,  0,  0,
	0,  0,  0,  0,  0,  0,  0,  0,  0,
	0,  0, 15, 15, 15,  0,  0,  0,  0,
	0,  0, 15, 15,  0,  0,  0,  0,  0,
	0,  0,  0,  0, 15,  0,  0,  0,  0,
	0,  0, 15,  0, 15,  0,  0,  0,  0,
	0,  0,  0, 15,  0,  0,  0,  0,  0,
	0,  0,  0,  0,  0,  0,  0,  0,  0,
	0,  0,  0,  0,  0,  0,  0,  0,  0,
	0,  0,  0, 15, 15,  0,  0,  0,  0,
	0,  0, 15,  0,  0,  0,  0,  0,  0,
	0,  0, 15, 15,  0,  0,  0,  0,  0,
	0,  0, 15,  0, 15,  0,  0,  0,  0,
	0,  0,  0, 15,  0,  0,  0,  0,  0,
	0,  0,  0,  0,  0,  0,  0,  0,  0,
	0,  0,  0,  0,  0,  0,  0,  0,  0,
	0,  0, 15, 15, 15,  0,  0,  0,  0,
	0,  0,  0,  0, 15,  0,  0,  0,  0,
	0,  0,  0, 15,  0,  0,  0,  0,  0,
	0,  0,  0, 15,  0,  0,  0,  0,  0,
	0,  0,  0, 15,  0,  0,  0,  0,  0,
	0,  0,  0,  0,  0,  0,  0,  0,  0,
	0,  0,  0,  0,  0,  0,  0,  0,  0,
	0,  0,  0, 15,  0,  0,  0,  0,  0,
	0,  0, 15,  0, 15,  0,  0,  0,  0,
	0,  0,  0, 15,  0,  0,  0,  0,  0,
	0,  0, 15,  0, 15,  0,  0,  0,  0,
	0,  0,  0, 15,  0,  0,  0,  0,  0,
	0,  0,  0,  0,  0,  0,  0,  0,  0,
	0,  0,  0,  0,  0,  0,  0,  0,  0,
	0,  0,  0, 15,  0,  0,  0,  0,  0,
	0,  0, 15,  0, 15,  0,  0,  0,  0,
	0,  0,  0, 15, 15,  0,  0,  0,  0,
	0,  0,  0,  0, 15,  0,  0,  0,  0,
	0,  0,  0,  0, 15,  0,  0,  0,  0,
};

const uint8_t img_allhi[5][15] PROGMEM = {
	16,16, 0, 0,16,16,16, 0, 0,16,16, 0,16,16,16,
	16, 0,16, 0,16, 0, 0, 0,16, 0, 0, 0, 0,16, 0,
	16,16, 0, 0,16,16, 0, 0, 0,16, 0, 0, 0,16, 0,
	16, 0,16, 0,16, 0, 0, 0, 0, 0,16, 0, 0,16, 0,
	16,16, 0, 0,16,16,16, 0,16,16, 0, 0, 0,16, 0,
};

const uint8_t img_noob[5][7] PROGMEM = {
	 0,  0,  0, 15,  0,  0,  0,
	 0,  0, 15,  0, 15,  0,  0,
	 0,  0, 15,  0, 15,  0,  0,
	 0,  0, 15,  0, 15,  0,  0,
	 0,  0,  0, 15,  0,  0,  0,
};

// Melodien definieren
const uint16_t start_melody[][2] PROGMEM =
{
  //------------ 1
  m_set_volume, LOUD,
  m_8_dot_note, m_D4,
  m_rest      , m_16_rest,
  m_8_dot_note, m_D4,
  m_rest      , m_16_rest,
  m_8_dot_note, m_D4,
  m_rest      , m_16_rest,
  m_8_note    , m_Bb3,
  m_rest      , m_16_rest,
  m_16_note   , m_F4,
  //---------- 2
  m_8_dot_note, m_D4,
  m_rest      , m_16_rest,
  m_8_note    , m_Bb3,
  m_rest      , m_16_rest,
  m_16_note   , m_F4,
  m_4_dot_note, m_D4,
  m_rest      , m_8_rest,
  m_stop      , 0
};

const uint16_t hiscore_melody[][2] PROGMEM =
{
  //------------ 1
  m_set_volume, LOUD,
  m_16_dot_note, m_E4,
  m_rest      , m_32_rest,
  m_16_dot_note, m_Ft4,
  m_rest      , m_32_rest,
  m_16_dot_note, m_E4,
  m_rest      , m_32_rest,
  m_16_dot_note, m_Ft4,
  m_rest      , m_32_rest,
  m_8_dot_note, m_D4,
  m_rest      , m_16_rest,
  m_16_note    , m_A4,
  m_rest      , m_16_rest,
  m_16_note   , m_A4,
  m_rest      , m_16_rest,
  m_4_note   , m_A4,
  m_stop      , 0
 };

const uint16_t gameover_melody[][2] PROGMEM =
{
  //------------ 1
  m_set_volume, LOUD,
  m_8_dot_note, m_E4,
  m_16_dot_note, m_D4,
  m_16_dot_note, m_C4,
  m_8_dot_note, m_B3,
  m_stop      , 0
 };


// Funktionen definieren
void soundplus() {  // Ton wird gespielt, wenn der Spieler Punkte hinzufügt, Länge 100ms
	gamekit.play_tone(440,50,LOUD);   // Spiele einen Ton mit 440 Herz für 50 Millisekunden laut ab
	delay(20);                       // Mache eine Pause für 20 Millisekunden
	gamekit.play_tone(576,30,SILENT);  // Spiele einen Ton mit 576 Herz für 30 Millisekunden leise ab
}

void soundminus() {  // Ton wird gespielt, wenn der Spieler Punkte löscht, Länge 100ms
	gamekit.play_tone(440,50,LOUD);    // Spiele einen Ton mit 440 Herz für 50 Millisekunden laut ab
	delay(20);                         // Mache eine Pause für 20 Millisekunden
	gamekit.play_tone(384,30,SILENT);  // Spiele einen Ton mit 384 Herz für 30 Millisekunden leise ab
}

void metronom() {	// Ton wird gespielt, wenn keine Taste gedrückt wurde, Länge 100ms
	gamekit.play_tone(220,10,LOUD);
	delay(110-level*10);
}

void rndchange(int color) {  // Diese Funktion ändert zufällig einen Punkt auf dem Bildschirm
	uint8_t rand1, rand2;   // Variablen für Funktion definieren
	rand1 = random(3) + 1;  // Wähle zwei zufällige Zahlen, um einen Punkt
	rand2 = random(7);      // auf dem Bildschirm zu wählen
	if (j = gamekit.get_pixel(rand1,rand2)) {  // Wenn der Punkt bereits leuchtet...
		if (j>12) {  // ...und heller als 12 ist...
		gamekit.set_pixel(rand1, rand2, (j-6)); // ... dann mache ihn um 6 dunkler
		}
		else {      // ... und nicht heller als 12 ist...
		gamekit.set_pixel(rand1, rand2, 0);    //  ... dann mache ihn ganz schwarz
		}
	}
	else {                                    // Wenn der Punkt nicht leuchtet...
		gamekit.set_pixel(rand1,rand2, (15-color));  // ... dann lasse ihn aufleuchten
	}
}

void savedisp() {                      // Speichere den aktuellen Bildschirm ab
	for (i = 0; i<5; i++) {        // save display in RAM (uses up RAM)!
		for (j=0; j < 7; j++) {
			img_afterboot[i][j] = gamekit.get_pixel(i,j);  // Jeder einzelne Punkt des Bildschirms wird durchgezählt und abgespeichert in img_afterboot
		}
	}
}

void loaddisp() {
	for (i = 0; i<5; i++) {        // Lade den letzten gespeicherten Bildschirm wieder auf den Bildschirm
		for (j=0; j < 7; j++) {
			gamekit.set_pixel(i,j,img_afterboot[i][j]);  // Jeder einzelne Punkt des Bildschirms wird durchgezählt und aus img_afterboot geladen
		}
	}
}

void clearoffset(int max) {   // Diese Funktion löscht die erste und die letzte Zeile des Bildschirms
	for (i= 0; i < max;i++) {
		gamekit.set_pixel(0,i,0);
		gamekit.set_pixel(4,i,0);
	}
}

void showoffset() {  // Diese Funktion schreibt das aktuelle Offet auf die erste oder letzte Zeile des Bildschirms
	if (offset > 8) {   // Wenn das Offset grösser als 8 ist, füllen wir die oberste Zeile
		for (i=8; i < offset;i++) {
			gamekit.set_pixel(0,(i-8),15);  // hier zeigen wir soviele Punkte an, wie leuchten sollen
			img_afterboot[0][(i-8)] = 15;  // und aktualisieren auch den Bildspeicher
			gamekit.set_pixel(4,(i-8),0); // und die unterste Zeile löschen wir
			img_afterboot[4][(i-8)] = 0;  // und wieder aktualisieren wir den Bildspeicher
		}
		for (i=offset; i < 15;i++) {
			gamekit.set_pixel(0,(i-8),0); // hier löschen wir alle Punkte, die nicht leuchten sollen
			img_afterboot[0][(i-8)] = 0;
			gamekit.set_pixel(4,(i-8),0); // und den Rest der untersten Zeile löschen wir
			img_afterboot[4][(i-8)] = 0;
		}
	}
	else if (offset < 8) { 
		for (i=1; i < offset;i++) { // Wenn das Offset kleiner als 8 ist, füllen wir die unterste Zeile
			gamekit.set_pixel(4,(7-i),0);  // hier füllen wir die unterste Zeile mit leuchtenden Punkten
			img_afterboot[4][(7-i)] = 0;
			gamekit.set_pixel(0,(7-i),0);  // und die oberste löschen wir
			img_afterboot[0][(7-i)] = 0;
		}
		for (i=offset; i < 8;i++) {
			gamekit.set_pixel(4,(7-i),15);  // hier füllen wir die unterste Zeile mit schwarzen Punkten
			img_afterboot[4][(7-i)] = 15;
			gamekit.set_pixel(0,(7-i),0);    // und die oberste löschen wir
			img_afterboot[0][(7-i)] = 0;
		}
	}
	else {  // Wenn das Offset gleich 8 ist, dann löschen wir beide Zeilen
		clearoffset(7);
	}
}

void showscore(uint8_t showlevel) {  // Diese Funktion zeigt die erreichte Punktzahl an, wenn ein Spiel zu Ende ist
	showlevel--;

	uint8_t scorepixels, scorerow=0;   // Variablen für Funktion definieren
	gamekit.load_image( img_empty );  // Der Bildschirm wird gelöscht
	gamekit.load_map( (uint8_t *) img_levels,63,9,(showlevel*7),2); //Auf der linken Bildschirmhälfte zeigen wir die Nummer des erreichten Levels

	scorepixels = (score-showlevel*150)/10 ;   // Für zehn erreichte Punkte wird ein heller Punkt angezeigt. Hier berechnen wir, wie viele das sind
	for (i = 0; i < scorepixels; i++) {   // Die berechnete Anzahl Pixel zeigen wir nun an
		if (i<3) {  // Pixel in der obersten Zeile
			gamekit.set_pixel(0,(i+4),15);  // Setze einen Pixel in Zeile 0, Spalte (i+4), mit Helligkeit 15
		}
		else if (i<6) { // Pixel in der zweiten Zeile
			gamekit.set_pixel(1,(i+1),15); // Setze einen Pixel in Zeile 1, Spalte (i+4), mit Helligkeit 15
			scorerow=1; // Zähle mit, in welcher Zeile wir sind
		}
		else if (i<9) { // Pixel in der dritten Zeile
			gamekit.set_pixel(2,(i-2),15); // Setze einen Pixel in Zeile 2, Spalte (i+4), mit Helligkeit 15
			scorerow=2; // Zähle mit, in welcher Zeile wir sind
		}
		else if (i<12) { // Pixel in der vierten Zeile
			gamekit.set_pixel(3,(i-5),15); // Setze einen Pixel in Zeile 3, Spalte (i+4), mit Helligkeit 15
			scorerow=3; // Zähle mit, in welcher Zeile wir sind
		}
		else if (i<15) { // Pixel in der fünften Zeile
			gamekit.set_pixel(4,(i-8),15); // Setze einen Pixel in Zeile 4, Spalte (i+4), mit Helligkeit 15
			scorerow=4; // Zähle mit, in welcher Zeile wir sind
		}
	}
	if (fmod(i,3) == 0) {
		if ((score-showlevel*150) < 10) {
			gamekit.set_pixel(0,4,(score-showlevel*150));
		}
		else {
			gamekit.set_pixel((scorerow+1),(i-scorerow*3+1),(score-showlevel*150-i*10));  // Der letzte Punkt zeigt die Punkte von 1-9 an
		}
	}
	else {
		gamekit.set_pixel(scorerow,(i-scorerow*3+4),(score-showlevel*150-i*10));  // Der letzte Punkt zeigt die Punkte von 1-9 an
	}
}

void masterscreen(){  // Diese Funktion spielt den Meister-Schluss-Bildschirm ab, wenn jemand das Spiel zu Ende spielt
	i = 0;
	while (!gamekit.button_pressed(butt_FUNCA)) { // Die Master-Laufschrift anzeigen, bis Knopf A gedrückt wird
		if (i==39) {    // Wenn die Laufschrift durchgelaufen ist, wieder von vorne beginnen
			i = 0;
		}
		gamekit.load_map((uint8_t *) img_master,5,39,0,i);
		soundplus();
		soundminus();
		delay(150);
		i++;
	}
	showscore(level);
}

void alltime_hi() {  // Diese Funktion vergleicht die Punktzahl mit der im EEPROM
	uint16_t alltime = eeprom_read_word(&allhi);
	if (hi > alltime) {  // Ist die Punktzahl die beste seit Installation?
		alltime = hi;
		eeprom_write_word(&allhi, alltime);  // Wenn ja, dann schreiben wir die neue Punktzahl ins EEPROM
		for (i=0; i<15; i++) {  // Und spielen die "BEST"-Laufschrift ab!
			gamekit.load_map((uint8_t *) img_allhi,5,15,0,i);
			gamekit.play_tone((440+i*40),50,LOUD);  // dazu ein netter Ton
			delay(150);
		}
	}
}

void gamestart() {  // Diese Funktion startet das Spiel
	gamekit.play_melody((uint16_t *) start_melody);  // Spiele die Start-Melodie
	score = 0;  // Setze alle Zähler auf Null
        offset = 8; // Offset = 8 heissst, dass wir im Gleichgewicht sind. 
	level = 0;
	gamekit.load_image(img_boot);  // Lade das Anfangsbild
	for (i = 0; i<20; i++) {    // Verändere das Anfangsbild zufällig über 20 Runden
		rndchange(i);
		delay(200);
	}
	delay(2000);  // Warte 2 Sekunden
	clearoffset(7);  // Lösche die oberste und die unterste Zeile
	savedisp();  // Speichere den aktuellen Bildschirm ab
}

uint8_t gameover() {  // Diese Funktion wir aufgerufen, wenn das Spiel zu Ende ist
	uint8_t buttons;
	noobi = 15;
	if (score > hi) {  // Haben wir eine neue Bestleistung?
		gamekit.play_melody((uint16_t *) hiscore_melody);  // Spiele High-Score Melodie ab
		gamekit.load_image( img_hi );  // Lade High Score Bild
		hi = score;  // Speichere die neue High Score ab
		delay(3000); // Zeige das Bild 3 Sekunden an
		alltime_hi();  // Und schaue nach, ob es eine ewige Bestleistung ist
	}
	else {  // Wir haben keine neue High Score
		gamekit.play_melody((uint16_t *) gameover_melody);  // Spiele Game Over Melodie ab
		gamekit.load_image( img_end );  // Lade das Game Over Bild
		delay(3000);  // Zeige das Bild für 3 Sekunden an
	}
	showscore(level);
	while (( buttons = gamekit.get_buttons()) == 0x3F) { ; }  // Warte, bis der Spieler einen Knopf drückt
	if (buttons == 0x3B) {  // Wurde A gedrückt, dann fangen wir im Noob-Modus an
		noobi = 0;
		noobstart();
	}
	if ((score != 0) || (offset != 8)) {  // Wurde gamestart erst gerade ausgeführt?
		gamestart();
	}
}

void levelplus() {  // Diese Funktion erhöht das Level
	gamekit.load_image(img_empty);
	if (level > 0 ) {  // Kommen wir mindestens in Level 2?
		for (i=(level-1)*7;i< (level*7+1);i++) {  // Scrolle de Levelzahl ein Level hoeher
			gamekit.load_map( (uint8_t *) img_levels,63,9,i,0);
			soundplus();
			soundminus();
		}
	}
	else {  // Zeige die Levelzahl 1
			gamekit.load_map( (uint8_t *) img_levels,63,9,0,0);
			soundplus();
			soundminus();
	}
	level++;
	delay(1000);
	loaddisp();
}

void nooblevel() {  // Diese Funktion zeigt die Null für das Noob-Level
	gamekit.load_image(img_empty);
	gamekit.load_image(img_noob);
	soundplus();
	soundminus();
	delay(1000);
	loaddisp();
}

void button_interaction() {
	uint8_t a,b,up,left,right,down;
	// Lese aus, welche Knöpfe gedrückt sind. Da die Knöpfe jeden Drchgang neu gedrückt werden sollen, können wir gamekit.get_buttons nicht nutzen
	a = gamekit.button_pressed(butt_FUNCA);
	b = gamekit.button_pressed(butt_FUNCB);
	up = gamekit.button_pressed(butt_UP);
	left = gamekit.button_pressed(butt_LEFT);
	right = gamekit.button_pressed(butt_RIGHT);
	down = gamekit.button_pressed(butt_DOWN);
	if (a) {
		if(up) {           // Ist der a und der nach oben-Knopf gedrückt?
			gamekit.set_pixel(1,random(7),15);  // Setze einen Pixel in der zweiten Zeile, in einer zufälligen Spalte, mit Helligkeit 15
			soundplus();
		}
		else if(down) {   // Ist der A und der nach unten-Knopf gedrückt?
			gamekit.set_pixel(3,random(7),15);  // Setze einen Pixel in der vierten Zeile, in einer zufälligen Spalte, mit Helligkeit 15
			soundplus();
		}
		else if(left) {   // Ist der A und der links-Knopf gedrückt? 
			gamekit.set_pixel((random(2)+1),random(3),15);  // Setze einen Pixel in der zweiten bis vierten Reihe, in der ersten bis dritten Spalte, mit Helligkeit 15
			soundplus();
		}
		else if(right) {  // Ist der A und der rechts-Knopf gedrückt?
			gamekit.set_pixel((random(2)+1),(random(3)+4),15);
			soundplus();
	}
	}
	else if (b) {
		if(up) {           // Ist der B und der nach oben-Knopf gedrückt?
			gamekit.set_pixel(1,random(7),0);  // lösche einen Pixel in der zweiten Zeile, in einer zufälligen Spalte, mit Helligkeit 0
			gamekit.set_pixel(1,random(7),0);
			gamekit.set_pixel(1,random(7),0);
			soundminus();
		}
		else if(down) {   // Ist der B und der nach unten-Knopf gedrückt?
			gamekit.set_pixel(3,random(7),0);
			gamekit.set_pixel(3,random(7),0);
			gamekit.set_pixel(3,random(7),0);
			soundminus();
		}
		else if(left) {   // Ist der B und der links-Knopf gedrückt? 
			gamekit.set_pixel((random(2)+1),random(3),0);
			gamekit.set_pixel((random(2)+1),random(3),0);
			gamekit.set_pixel((random(2)+1),random(3),0);
			soundminus();
		}
		else if(right) {  // Ist der B und der rechts-Knopf gedrückt?
			gamekit.set_pixel((random(2)+1),(random(3)+4),0);
			gamekit.set_pixel((random(2)+1),(random(3)+4),0);
			gamekit.set_pixel((random(2)+1),(random(3)+4),0);
			soundminus();
		}
	}
	else {                  // Wenn A oder B nicht gedrückt wurden, spiele den Metronom-Ton ab
		metronom();
	}
}

void count_pixel() {
	uint8_t img_pixel = 0;
	for (i = 1; i<4; i++) {  // Speichere Spielfläche im Arbeitsspeicher (RAM) und zähle leuchtende Punkte
		for (j=0; j < 7; j++) {
			if (img_afterboot[i][j] = gamekit.get_pixel(i,j)) { // Leuchtet der gespeicherte Punkt?
				img_pixel = img_pixel++;  // Wenn ja, Punkt zählen
			}
		}
	}
	if (img_pixel > 10) {  // Wenn mehr als 10 Punkte leuchten...
		offset++;  // ... zählen wir eins zum Offset dazu
	}
	else if (img_pixel < 9) {  // Wenn weniger als 9 Punkte leuchten...
		offset--;  // .. zählen wir eins vom Offset ab
	}
}


void noobstart() {  // Diese Funktion startet das Neulings-Spiel
	gamekit.play_melody((uint16_t *) start_melody);  // Spiele die Start-Melodie
	score = 0;  // Setze alle Zähler auf Null
        offset = 8;
	level = 0;
	gamekit.load_image(img_boot);  // Lade das Anfangsbild
	for (i = 0; i<20; i++) {    // Verändere das Anfangsbild zufällig über 20 Runden
		rndchange(i);
		delay(200);
	}
	delay(2000);  // Warte 2 Sekunden
	clearoffset(7);  // Lösche die oberste und die unterste Zeile
	savedisp();  // Speichere den aktuellen Bildschirm ab
	nooblevel();
	for (noobi = 0; noobi < 15; noobi++) {  // Jetzt darf der Neuling 15 Runden üben
		button_interaction();
		for (i=0; i<3; i++){              // Bleibe drei runden in der Schleife
			delay(250);         // Warte 250 Millisekunden
			rndchange(random(15));    // Wechsle einen Punkt von hell auf dunkel oder von dunkel auf hell
		}
		// game 'housekeeping'
		count_pixel();
		if ((offset == 16) || (offset == 0)) {  // Wenn das Offset auf 16 oder auf 0 geht, dann ist das Spiel zu Ende!
			level = 1;
			score = 0;
			gameover();
			break;
		}
		else {
			showoffset();

		}
	}
}

/**************************************************************************

Unten werden die Funktionen setup() und loop() definiert
Diese braucht jedes Programm für das Mignon Gamekit
setup() wird beim einschalten des Gamekits einmal ausgeführt
loop() wird danach immer wiederholt, bis das Gamekit abgeschaltet wird

**************************************************************************/

void setup(){
	uint8_t buttons;
	Serial.begin(19200);
	gamekit.Begin();
	gamekit.set_button_timing(0,0);
	hi = 10;
        offset = 0;
	i = 0;
	uint16_t allcheck = eeprom_read_word(&allhi);  // dieser Block schreibt die eeprom-Highscore
	if (allcheck == 65535) {                       // neu, falls sie beim laden des Codes 
		allcheck = 0;                          // vom Arduino-Environment nicht geschrieben
		eeprom_write_word(&allhi, allcheck);   // wurde
	}
	while ((buttons = gamekit.get_buttons()) == 0x3F) {   // Die Intro-Laufschrift anzeigen, bis irgend ein Knopf gedrückt wird
		if (i==89) {    // Wenn die Laufschrift durchgelaufen ist, wieder von vorne beginnen
			i = 0;
		}
		gamekit.load_map((uint8_t *) img_intro, 5, 89, 0, i);
		delay(150);
		i++;
	}
	savedisp();   // Wir speichern den Bildschirm, um die Pixelmap zu intialisieren
	if (buttons == 0x3B) {  // Wurde A gedrückt, dann fangen wir im Noob-Modus an
		noobstart();
	}
	if ((score != 0) || (offset != 8)) {  // Wurde gamestart erst gerade ausgeführt?
		gamestart();  // Wir starten das Spiel
	}
}

void loop(){	// Das Spiel läuft immer in dieser Schleife
		Serial.print("game: offset = ");
		Serial.println(offset, DEC);

		if (score > 1349) {                // Nach Level neun das Spiel beenden
			masterscreen();
			gamestart();
		}
		if (score >= (level)*150) {                 // Level alle 150 Punkte erhöhen
			levelplus();
		} 
		button_interaction();

		for (i=0; i<3; i++){              // Bleibe drei runden in der Schleife
			delay(100/level);         // Warte 100 Millisekunden geteilt durch das Level
			rndchange(random(15));    // Wechsle einen Punkt von hell auf dunkel oder von dunkel auf hell
		}
		// game 'housekeeping'
		count_pixel();
		if ((offset == 16) || (offset == 0)) {  // Wenn das Offset 16 oder 0 ist, dann ist das Spiel zu Ende!
			gameover();
		}
		else {
			showoffset();
		}
	score = score++;  // Für jede gespielte Runde gibt es einen Spielpunkt mehr
//	offset = 0; //Nur fuer testzwecke auskommentieren!
}
